package com.galaxis.manatee.service.impl;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunGroupProjectDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.exception.BigSizeException;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.ChuanyunService;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import com.galaxis.manatee.task.ChuanyunHumanResourceScheduledTask;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

@Slf4j
@Service
public class ChuanyunServiceImpl implements ChuanyunService {

    private final Integer pageSize=20;
    private final ChuanyunHumanResourceScheduledTask chuanyunHumanResourceScheduledTask;
    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;

    public ChuanyunServiceImpl(ChuanyunHumanResourceScheduledTask chuanyunHumanResourceScheduledTask, ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask, ChuanYunManager chuanYunManager, ChuanyunGroupProjectDao chuanyunGroupProjectDao) {
        this.chuanyunHumanResourceScheduledTask = chuanyunHumanResourceScheduledTask;
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
        this.chuanYunManager = chuanYunManager;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
    }

    @Override
    public void updateProjectMember() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {

                    //处理项目对象中的项目成员数据(改为手动调用处理)
                    GroupProjectUpdateDTO groupProjectUpdateDTO= new GroupProjectUpdateDTO();
                    chuanyunBasicDataScheduledTask.projectMemberCheck(chuanyunGroupProjectDO);
                    try {
                        groupProjectUpdateDTO=new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
                        var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                        ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                        if(chuanyunSaveDTO.getSuccessful()){
                            log.info(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员成功");

                        }else{
                            log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目组成员失败");
                            log.warn(chuanyunGroupProjectDO+"");
                        }
                    } catch (BigSizeException e) {
                        e.printStackTrace();
                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                    }
                });
                chuanyunGroupProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    class GroupProjectUpdateDTO{
//        /**
//         * 项目总监
//         * U9项目责任人
//         */
//        @JsonProperty(value = "F0000006")
//        private String projectOwner;
//
//        /**
//         * IBG项目经理
//         */
//        @JsonProperty("F0000012")
//        private String ibgProjectManager;
//
//        /**
//         * EBG项目经理
//         */
//        @JsonProperty("F0000011")
//        private String ebgProjectManager;

        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberList;

//        /**
//         * 项目组织信息
//         * TODO 这个DTO是用于更新氚云数据的，不需要使用@OneToMany注解吧？使用目的是啥
//         */
//        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true,fetch = FetchType.EAGER)
//        @JsonProperty("D001789Fd992bf8fbe114c9babe0f98cf676f8a2")
////        @JsonProperty("D001789Fvspc68z01idftf5nqt66pkuc0")
//        private List<ChuanyunGroupProjectDetailDO> detailList;

        public GroupProjectUpdateDTO() {
        }

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
//                if(null!=chuanyunGroupProjectDO.getProjectOwnerObject()){
//                    this.projectOwner=chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId();
//                }
//                if(null!=chuanyunGroupProjectDO.getIbgProjectManagerObject()){
//                    this.ibgProjectManager=chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId();
//                }
//                if(null!=chuanyunGroupProjectDO.getEbgProjectManagerObject()){
//                    this.ebgProjectManager=chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId();
//                }
                //TODO 更新项目经理时更新项目组织信息会是这两个业务流程耦合，不方便后续维护需要考虑单独一个DTO作为更新项目组织信息的对象
                if(null!=chuanyunGroupProjectDO.getProjectMemberList()){
                    this.projectMemberList =chuanyunGroupProjectDO.getProjectMemberList();
                }
//                if(null!=chuanyunGroupProjectDO.getDetailList()){
//                    this.detailList=chuanyunGroupProjectDO.getDetailList();
//                }
            }
        }
    }
}
